<?php
/*
Plugin Name:  Builder Infinite Background
Plugin URI:   https://themify.me/addons/infinite-background
Version:      3.5.2
Description:  This Builder addon allows you to set infinite scrolling row background image either horizontally or vertically in Row > Options. It requires to use with the latest version of any Themify theme or the Themify Builder plugin.
Author:       Themify
Author URI:   https://themify.me/
Text Domain:  builder-infinite-background
Domain Path:  /languages
Requires PHP: 7.2
Compatibility: 7.0.0
*/

class Builder_Infinite_Background {

	private const SPEED = 5;
	
	/**
	 * init
	 *
	 * @access	public
	 * @return	void
	 */
	public static function init() {
		if(is_admin()){
			add_filter( 'plugin_row_meta', array( __CLASS__, 'themify_plugin_meta'), 10, 2 );
			add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), array( __CLASS__, 'action_links') );
		}
		// Front end control
		add_action('themify_builder_setup_modules', array(__CLASS__, 'run'));
	}
	
	public static function get_version():string{
		return '3.5.2';
	}

	public static function run() {
		add_action( 'themify_builder_background_styling', array( __CLASS__, 'do_infinite_bg' ), 10, 3 );
		add_filter( 'themify_builder_active_vars', [ __CLASS__, 'builder_active_enqueue' ] );
	}

	public static function themify_plugin_meta(array $links,string $file ):array {
		if ( plugin_basename( __FILE__ ) === $file ) {
			$row_meta = array(
			  'changelogs'    => '<a href="' . esc_url( 'https://themify.org/changelogs/' ) . basename( dirname( $file ) ) .'.txt" target="_blank" aria-label="' . esc_attr__( 'Plugin Changelogs', 'themify' ) . '">' . esc_html__( 'View Changelogs', 'themify' ) . '</a>'
			);
	 
			return array_merge( $links, $row_meta );
		}
		return (array) $links;
	}

	public static function action_links(array $links ):array {
		if ( is_plugin_active( 'themify-updater/themify-updater.php' ) ) {
			$tlinks = array(
			 '<a href="' . admin_url( 'index.php?page=themify-license' ) . '">'.__('Themify License', 'themify') .'</a>',
			 );
		} else {
			$tlinks = array(
			 '<a href="' . esc_url('https://themify.me/docs/themify-updater-documentation') . '">'. __('Themify Updater', 'themify') .'</a>',
			 );
		}
		return array_merge( $links, $tlinks );
	}

	public static function builder_active_enqueue(array $vars ):array {
	    $url = trailingslashit( plugin_dir_url( __FILE__ ) ). 'assets/active.js';
		if(!isset($vars['addons'])){//backward
			themify_enque_script( 'tb_builder-infinite-background', $url, self::get_version(), [ 'themify-builder-app-js' ] );
		}
		else{
			$vars['addons'][$url]=self::get_version();
		}

		$i18n = include( trailingslashit( __DIR__ ) . 'includes/i18n.php' );
		$vars['i18n']['label'] = array_merge( $i18n, $vars['i18n']['label'] );
		$vars['infiniteBgVars'] = [
			'horizontal' => self::get_bg_type( 'horizontally' ),
            'vertical' => self::get_bg_type( 'vertically' ),
            'speed' => self::SPEED
		];

		return $vars;
	}

	public static function do_infinite_bg( $builder_id,$row,$type  ) {
		if ((isset($row['element_id']) && (!isset($row['styling']['background_type']) || (isset($row['styling']['background_type']) && $row['styling']['background_type']==='image')) && !empty($row['styling']['row_scrolling_background']) && $row['styling']['row_scrolling_background'] !== 'disable' &&  !empty($row['styling']['background_image']))) {
			if(empty($row['styling']['row_scrolling_background_speed'])){
				$row['styling']['row_scrolling_background_speed'] = self::SPEED;
			}
			$speed = $row['styling']['row_scrolling_background_speed'] . 's';
			$replace = array('#speed#'=>$speed,'#unique_id#'=>$row['element_id']);
			$template = null;
			if ( $row['styling']['row_scrolling_background'] === 'bg-scroll-horizontally' ) {
				if(!empty($row['styling']['row_scrolling_background_width'])){
					$template = self::get_bg_type('horizontally');
					$replace['#width#'] = $row['styling']['row_scrolling_background_width'];
				}

			} 
			elseif(!empty($row['styling']['row_scrolling_background_height'])) {
				$template = self::get_bg_type('vertically');
				$replace['#height#'] = $row['styling']['row_scrolling_background_height'];
			}
			if($template!==null){
				$template = strtr($template,$replace);
				echo '<style>', '.tb_'.$row['element_id'] , $template,'</style>';
			}
		}

	}
        
    private static function get_bg_type(string $type): string{

		$return='{animation:bg-animation-'.$type.'-#unique_id# #speed# linear infinite!important;background-repeat:repeat!important}';
		if($type==='horizontally'){
			$return.='@keyframes bg-animation-horizontally-#unique_id#{
							from{background-position:0 0}
							to{background-position:-#width#px 0}
					}';
		}
		else{
			$return.='@keyframes bg-animation-vertically-#unique_id#{
				from{background-position:0 0}
				to{background-position:0% -#height#px}
		}';
		}
		return $return;
	}
}
Builder_Infinite_Background::init();
